package java.awt.geom;

import java.io.Serializable;

public abstract class Point2D implements Cloneable
{
    protected Point2D() {
    }
    
    public abstract double getX();
    
    public abstract double getY();
    
    public abstract void setLocation(double p0, double p1);
    
    public void setLocation(Point2D point2D) {
        this.setLocation(point2D.getX(), point2D.getY());
    }
    
    public static double distanceSq(double n, double n2, double n3, double n4) {
        n -= n3;
        n2 -= n4;
        return n * n + n2 * n2;
    }
    
    public static double distance(double n, double n2, double n3, double n4) {
        n -= n3;
        n2 -= n4;
        return Math.sqrt(n * n + n2 * n2);
    }
    
    public double distanceSq(double n, double n2) {
        n -= this.getX();
        n2 -= this.getY();
        return n * n + n2 * n2;
    }
    
    public double distanceSq(Point2D point2D) {
        double n = point2D.getX() - this.getX();
        double n2 = point2D.getY() - this.getY();
        return n * n + n2 * n2;
    }
    
    public double distance(double n, double n2) {
        n -= this.getX();
        n2 -= this.getY();
        return Math.sqrt(n * n + n2 * n2);
    }
    
    public double distance(Point2D point2D) {
        double n = point2D.getX() - this.getX();
        double n2 = point2D.getY() - this.getY();
        return Math.sqrt(n * n + n2 * n2);
    }
    
    public Object clone() {
        try {
            return super.clone();
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
    
    @Override
    public int hashCode() {
        long n = java.lang.Double.doubleToLongBits(this.getX()) ^ java.lang.Double.doubleToLongBits(this.getY()) * 31L;
        return (int)n ^ (int)(n >> 32);
    }
    
    @Override
    public boolean equals(Object obj) {
        if ((obj instanceof Point2D)) {
            Point2D point2D = (Point2D)obj;
            return this.getX() == point2D.getX() && this.getY() == point2D.getY();
        }
        return super.equals(obj);
    }
    
    public static class Float extends Point2D implements Serializable
    {
        public float x;
        public float y;
        private static final long serialVersionUID = -2870572449815403710L;
        
        public Float() {
        }
        
        public Float(float x, float y) {
            this.x = x;
            this.y = y;
        }
        
        @Override
        public double getX() {
            return (double)this.x;
        }
        
        @Override
        public double getY() {
            return (double)this.y;
        }
        
        @Override
        public void setLocation(double n, double n2) {
            this.x = (float)n;
            this.y = (float)n2;
        }
        
        public void setLocation(float x, float y) {
            this.x = x;
            this.y = y;
        }
        
        @Override
        public String toString() {
            return "Point2D.Float[" + this.x + ", " + this.y + "]";
        }
    }
    
    public static class Double extends Point2D implements Serializable
    {
        public double x;
        public double y;
        private static final long serialVersionUID = 6150783262733311327L;
        
        public Double() {
        }
        
        public Double(double x, double y) {
            this.x = x;
            this.y = y;
        }
        
        @Override
        public double getX() {
            return this.x;
        }
        
        @Override
        public double getY() {
            return this.y;
        }
        
        @Override
        public void setLocation(double x, double y) {
            this.x = x;
            this.y = y;
        }
        
        @Override
        public String toString() {
            return "Point2D.Double[" + this.x + ", " + this.y + "]";
        }
    }
}
